"""here we're creating the Hel Man key exchange"""

from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives.asymmetric import dh
from cryptography.hazmat.primitives.kdf.hkdf import HKDF

parameters = dh.generate_parameters(generator=2, key_size=2048)
server_private_key = parameters.generate_private_key()
peer_private_key = parameters.generate_private_key()


def derived_key():
    """function for sending derived_key"""
    shared_key = server_private_key.exchange(peer_private_key.public_key())
    derived_key = HKDF(
        algorithm=hashes.SHA256(),
        length=32,
        salt=None,
        info=b'handshake data',
    ).derive(shared_key)

    return derived_key


def same_derived_key():
    """function for sending same_derived_key"""
    same_shared_key = peer_private_key.exchange(
        server_private_key.public_key()
    )
    same_derived_key = HKDF(
        algorithm=hashes.SHA256(),
        length=32,
        salt=None,
        info=b'handshake data',
    ).derive(same_shared_key)

    return same_derived_key


def comparing_key():
    """comparing 2 keys"""
    if derived_key() == same_derived_key():
        print("same")



from sanic.views import HTTPMethodView
from sanic.response import json
from sanic.exceptions import SanicException
from sanic.log import logger
import sqlite3
import subprocess
import base64

from camera import camera_on_cv, stop_camera
from delivered_key import derived_key, same_derived_key
from sdk_functions import change_led, busser_beep, get_media

connection = sqlite3.connect('sdk_db.db')
cursor = connection.cursor()

cursor.execute('''CREATE TABLE IF NOT EXISTS sdk_table (id INT,request_id VARCHAR NOT NULL, is_file_created BOOLEAN , 
        file_name TEXT, created_at DATETIME default CURRENT_TIMESTAMP, request_by TEXT, status BOOLEAN )''')


class SetUpRoutes(HTTPMethodView):
    """here the basic api method of get all data and create data"""

    async def get(self, request):
        """get all data from the table sdk_table"""
        select_all = "SELECT * FROM sdk_table"
        try:
            rows = cursor.execute(select_all).fetchall()
        except sqlite3.OperationalError as err:
            logger.error(f"sqlite3 OperationalError {str(err)} while get all data")
            return json(
                {
                    "status": "failed",
                    "message": str(err)
                }
            )
        field_names = [i[0] for i in cursor.description]
        row_list = []
        for row in rows:
            students_dict = dict(zip(field_names, row))
            row_list.append(students_dict)
        logger.info('Successfully get all data')

        return json({
            "data": row_list
        })

    async def post(self, request):
        """inserting data to table sdk_table by direct insert sql query"""
        data = request.json
        insert_records = f"INSERT INTO sdk_table (id,request_id, is_file_created, file_name, created_at, \
                            request_by, status) values(?,?,?,?,?,?,?) "
        try:
            contents = data['id'], data['request_id'], data['is_file_created'], data['file_name'], data['created_at'], \
                       data['request_by'], data['status']
        except KeyError as err:
            logger.error(f"KeyError,this {str(err)} field is missing while add data")
            return json(
                {
                    "status": "failed",
                    "message": f"{str(err)} missing"
                }
            )
        try:
            cursor.execute(insert_records, contents)
        except sqlite3.OperationalError as err:
            logger.error(f"sqlite3 OperationalError {str(err)}")
            return json(
                {
                    "status": "failed",
                    "message": str(err)
                }
            )
        connection.commit()
        logger.info('Successfully data added')
        return json({
            "data": data
        })


class IdRoutes(HTTPMethodView):
    """herre having the basic api method as getting data by Id , Update by Id,\
    incase needed for delete by id, update single field by id"""

    async def get(self, request, id_):
        """get data by id"""
        select_all = f"SELECT * FROM sdk_table WHERE id={id_}"
        try:
            rows = cursor.execute(select_all).fetchall()
        except sqlite3.OperationalError as err:
            logger.error(f"sqlite3 OperationalError {str(err)} while get data of id {id_}")
            return json(
                {
                    "status": "failed",
                    "message": str(err)
                }
            )
        field_names = [i[0] for i in cursor.description]
        row_list = []
        for row in rows:
            students_dict = dict(zip(field_names, row))
            row_list.append(students_dict)
        logger.info(f'Successfully get data of {id_}')
        return json({
            "data": row_list
        })

    async def put(self, request, id_):
        """update entire row by id in direct sql query Update"""
        data = request.json
        select_all = f"Update sdk_table set id=?,request_id=?,is_file_created=?,\
                        file_name=?,created_at=?,request_by=?,\
                        status=? where id=?"
        try:
            contents = data['id'], data['request_id'], data['is_file_created'], data['file_name'], data['created_at'], \
                       data['request_by'], data['status'], id_
        except KeyError as err:
            logger.error(f"KeyError,this {str(err)} field is missing while update in {id_}")
            return json(
                {
                    "status": "failed",
                    "message": f"{str(err)} missing"
                }
            )
        try:
            cursor.execute(select_all, contents)
        except sqlite3.OperationalError as err:
            logger.error(f"sqlite3 OperationalError {str(err)} while update in {id_}")
            return json(
                {
                    "status": "failed",
                    "message": str(err)
                }
            )
        connection.commit()
        logger.info(f'Successfully update data of {id_} as fully')
        return json({
            "data": data
        })

    async def delete(self, id_):
        """delete data by id"""
        select_all = f"DELETE from sdk_table where id=?"
        contents = id_
        try:
            cursor.execute(select_all, contents)
        except sqlite3.OperationalError as err:
            logger.error(f"sqlite3 OperationalError {str(err)} while delete {id_}")
            return json(
                {
                    "status": "failed",
                    "message": str(err)
                }
            )
        connection.commit()
        logger.info(f'Successfully delete data of {id_}')
        return json({
            "data": "successfully deleted"
        })

    async def patch(self, request, id_):
        """update a single field by id on by giving"""
        data = request.json
        key = list(data.keys())[0]
        value = list(data.values())[0]
        select_all = f"Update sdk_table set {key}=? where id=?"
        try:
            contents = value, id_
        except KeyError as err:
            logger.error(f"KeyError,this {str(err)} field is missing while update {key}")
            return json(
                {
                    "status": "failed",
                    "message": f"{str(err)} missing"
                }
            )
        try:
            cursor.execute(select_all, contents)
        except sqlite3.OperationalError as err:
            logger.error(f"sqlite3 OperationalError {str(err)} wihile update by field")
            return json(
                {
                    "status": "failed",
                    "message": str(err)
                }
            )
        connection.commit()
        logger.info(f'Successfully update data of {id_} in {key}')
        return json({
            "data": data
        })


class RequestIdRoutes(HTTPMethodView):
    """class function for Request Id"""

    async def get(self, request, request_id):
        """filter data on based by request_id from table"""
        select_all = f"SELECT * FROM sdk_table WHERE request_id='{request_id}'"
        try:
            rows = cursor.execute(select_all).fetchall()
        except sqlite3.OperationalError as err:
            logger.error(f"sqlite3 OperationalError {str(err)} in get request_id")
            return json(
                {
                    "status": "failed",
                    "message": str(err)
                }
            )
        field_names = [i[0] for i in cursor.description]
        row_list = []
        for row in rows:
            students_dict = dict(zip(field_names, row))
            row_list.append(students_dict)
        logger.info(f'Successfully get all data by request_id is {request_id}')
        return json({
            "data": row_list
        })


class StatusRoutes(HTTPMethodView):
    """class function for Status"""

    async def get(self, request, status):
        """filtering data on based status"""
        select_all = f"SELECT * FROM sdk_table WHERE status={status}"
        try:
            rows = cursor.execute(select_all).fetchall()
        except sqlite3.OperationalError as err:
            logger.error(f"sqlite3 OperationalError {str(err)}in get status")
            return json(
                {
                    "status": "failed",
                    "message": str(err)
                }
            )
        field_names = [i[0] for i in cursor.description]
        row_list = []
        for row in rows:
            students_dict = dict(zip(field_names, row))
            row_list.append(students_dict)
            logger.info(f'Successfully get all data by status is {status}')
        return json({
            "data": row_list
        })


class CreatedAtRoutes(HTTPMethodView):
    """class function for created at"""

    async def get(self, request, created_at):
        """filtering data on based created_at"""
        select_all = f"SELECT * FROM sdk_table WHERE created_at='{created_at}'"
        try:
            rows = cursor.execute(select_all).fetchall()
        except sqlite3.OperationalError as err:
            logger.error(f"sqlite3 OperationalError {str(err)}in get created_at")
            return json(
                {
                    "status": "failed",
                    "message": str(err)
                }
            )
        field_names = [i[0] for i in cursor.description]
        row_list = []
        for row in rows:
            students_dict = dict(zip(field_names, row))
            row_list.append(students_dict)
        logger.info(f'Successfully get all data by created_at is {created_at}')
        return json({
            "data": row_list
        })


class CameraOnRoutes(HTTPMethodView):
    """class function for open camera"""

    async def get(self, request):
        """filtering data on based created_at"""
        camera_on_cv()

        return json(
            {
                "status": "success",
            }
        )


class CameraCloseRoutes(HTTPMethodView):
    """class function for open camera"""

    async def get(self, request):
        """filtering data on based created_at"""
        stop_camera()

        return json(
            {
                "status": "camera closed",
            }
        )


class ImportCLanguage(HTTPMethodView):
    """class function for open camera"""

    async def get(self, request):
        """filtering data on based created_at"""
        file = request.form.get("file")
        subprocess.call(["gcc", f'{file}'])
        subprocess.call("./a.out")
        logger.info("c file is successfully read")

        return json(
            {
                "status": "file upload",
            }
        )


class ChangeLedRoutes(HTTPMethodView):
    """class function for change led"""

    async def get(self, request):
        """filtering data on based created_at"""
        change_led()
        logger.info("change_led is successfully done")

        return json(
            {
                "status": "success",
                "message": "Led change"
            }
        )


class BusserBeepRoutes(HTTPMethodView):
    """class function for busser_beep"""

    async def get(self, request):
        """filtering data on based created_at"""
        busser_beep()
        logger.info("busser_beep is successfully done")

        return json(
            {
                "status": "success",
                "message": "busser_beepe"
            }
        )


class GetMediaRoutes(HTTPMethodView):
    """class function for change led"""

    async def get(self, request):
        """filtering data on based created_at"""
        get_media()
        logger.info("get_media is successfully done")

        return json(
            {
                "status": "success",
                "message": "get_media"
            }
        )


class DerivedKeyRoutes(HTTPMethodView):
    """class function for sending derived key"""

    async def get(self, request):
        """filtering data on based created_at"""
        # derived_key()
        encoded = base64.b64encode(derived_key())
        logger.info("derived_key is successfully send")

        return json(
            {
                "status": "success",
                "data": str(encoded)
            }
        )


class SameDerivedKeyRoutes(HTTPMethodView):
    """class function for sending derived key"""

    async def get(self, request):
        """filtering data on based created_at"""
        # same_derived_key()
        encoded = base64.b64encode(same_derived_key())
        logger.info("same_derived_key is successfully given")

        return json(
            {
                "status": "success",
                "data": str(encoded)
            }
        )



"""open cv function for camera on"""

import cv2
import os
from sanic.log import logger

import time

filename = 'video.avi'
frames_per_second = 10
res = '720p'


def change_res(cap, width, height):
    cap.set(3, width)
    cap.set(4, height)


# Standard Video Dimensions Sizes
STD_DIMENSIONS = {
    "480p": (640, 480),
    "720p": (1280, 720),
    "1080p": (1920, 1080),
    "4k": (3840, 2160),
}


# grab resolution dimensions and set video capture to it.
def get_dims(cap, res='1080p'):
    width, height = STD_DIMENSIONS["480p"]
    if res in STD_DIMENSIONS:
        width, height = STD_DIMENSIONS[res]
    ## change the current caputre device
    ## to the resulting resolution
    change_res(cap, width, height)
    return width, height


# Video Encoding, might require additional installs
# Types of Codes: http://www.fourcc.org/codecs.php
VIDEO_TYPE = {
    'avi': cv2.VideoWriter_fourcc(*'XVID'),
    # 'mp4': cv2.VideoWriter_fourcc(*'H264'),
    'mp4': cv2.VideoWriter_fourcc(*'XVID'),
}

palm_cascade = cv2.CascadeClassifier('palm.xml')


# face_cascade = cv2.CascadeClassifier('haarcascade_frontalface_default.xml')
#
#
#
def get_video_type(filename):
    filename, ext = os.path.splitext(filename)
    if ext in VIDEO_TYPE:
        return VIDEO_TYPE[ext]
    return VIDEO_TYPE['avi']


#
GREEN = (0, 255, 0)
fonts = cv2.FONT_HERSHEY_COMPLEX


def camera_on_cv():
    """function for open camera in system and can end by pressing ' q ' key"""
    cap = cv2.VideoCapture(0)
    if not (cap.isOpened()):
        logger.error("camera is not opened")

    logger.info("camera is working well")
    out = cv2.VideoWriter(filename, get_video_type(filename), frames_per_second, get_dims(cap, res))
    prev = time.time()

    while True:
        ret, frame = cap.read()
        gray = cv2.cvtColor(frame, cv2.COLOR_BGR2GRAY)
        palm = palm_cascade.detectMultiScale(
            gray,
            scaleFactor=1.1,
            minNeighbors=5,
            minSize=(30, 30)
        )

        for (x, y, w, h) in palm:
            cv2.rectangle(frame, (x, y), (x + w, y + h), (0, 255, 0), 2)
            out.write(frame)

        cv2.putText(
            frame, f"Please show Your Palm", (30, 35),
            fonts, 0.6, GREEN, 2)

        cv2.imshow('frame', frame)
        cur = time.time()
        if cur - prev >= 30:
            cap.release()
            out.release()
            cv2.destroyAllWindows()
            break
        if cv2.waitKey(1) & 0xFF == ord('q'):
            logger.info("camera is closed")
            cap.release()
            out.release()
            cv2.destroyAllWindows()
            break


def stop_camera():
    """function for stop camera"""
    cap = cv2.VideoCapture(0)
    out = cv2.VideoWriter(filename, get_video_type(filename), frames_per_second, get_dims(cap, res))
    logger.info("camera is closed")
    cap.release()
    out.release()
    cv2.destroyAllWindows()
    del cap



"""sdk functions"""


def change_led():
    """function for change LED """
    response = "lead change"
    print(response)
    return response


def busser_beep():
    """function for Busser beep"""
    response = "busser beep"
    print(response)
    return response


def get_media():
    """function for get media"""
    response = "get media"
    print(response)
    return response



import json

import pytest
from sanic import Sanic, response
from http import HTTPStatus
import requests

payload = {
    "id": 10,
    "request_id": "SDT10",
    "is_file_created": True,
    "file_name": "Ayush",
    "created_at": "16-06-2022",
    "request_by": "Ashique",
    "status": False
}

payload_update = {
    "id": 5,
    "request_id": "SDT32",
    "is_file_created": False,
    "file_name": "Ayush",
    "created_at": "16-06-2022",
    "request_by": "Ashique",
    "status": False
}

is_file_created_missing = {
    "id": 6,
    "request_id": "SDT03",
    "file_name": "Ayush",
    "created_at": "16-06-2022",
    "request_by": "Ashique",
    "status": False
}

request_id_missing = {
    "id": 7,
    "is_file_created": True,
    "file_name": "Ayush",
    "created_at": "16-06-2022",
    "request_by": "Ashique",
    "status": False
}

request_by_missing = {
    "id": 5,
    "request_id": "SDT03",
    "is_file_created": True,
    "file_name": "Ayush",
    "created_at": "16-06-2022",
    "status": False
}
status_missing = {
    "id": 5,
    "request_id": "SDT03",
    "is_file_created": True,
    "file_name": "Ayush",
    "created_at": "16-06-2022",
    "request_by": "Ashique"
}

@pytest.fixture
def app():
    sanic_app = Sanic("TestSanic")

    @sanic_app.get("/")
    def basic(request):
        return response.text("foo")

    return sanic_app


def test_basic_test_client(app):
    request, response = app.test_client.get("/")

    assert request.method.lower() == "get"
    assert response.body == b"foo"
    assert response.status == 200


def test_get_all(app):
    """test getting all data"""
    request, response = app.test_client.get("http://127.0.0.1:5823/gets")
    assert response.status == 200

def test_post_data(app):
    """test post data"""
    request, response = app.test_client.post("http://127.0.0.1:5823/gets", data=json.dumps(payload))
    assert response.status == 200

def test_get_by_id(app):
    """test get by id"""
    request, response = app.test_client.get("http://127.0.0.1:5823/get/5")
    assert response.status == 200

def test_update_by_id(app):
    """test getting all data"""
    request, response = app.test_client.put("http://127.0.0.1:5823/get/5", data=json.dumps(payload_update))
    assert response.status == 200


def test_post_is_file_created_missing(app):
    """test post missing by is_file_created_missing"""
    request, response = app.test_client.post("http://127.0.0.1:5823/gets", data=json.dumps(is_file_created_missing))
    assert response.status_code == HTTPStatus.BAD_REQUEST

def test_post_request_id_missing(app):
    """test post missing by request_id_missing"""
    request, response = app.test_client.post("http://127.0.0.1:5823/gets", data=json.dumps(request_id_missing))
    assert response.status_code == HTTPStatus.BAD_REQUEST

def test_post_request_by_missing(app):
    """test post missing by request_id_missing"""
    request, response = app.test_client.post("http://127.0.0.1:5823/gets", data=json.dumps(request_id_missing))
    assert response.status_code == HTTPStatus.BAD_REQUEST

def test_post_status_missing(app):
    """test post missing by status"""
    request, response = app.test_client.post("http://127.0.0.1:5823/gets", data=json.dumps(status_missing))
    assert response.status_code == HTTPStatus.BAD_REQUEST


def test_busser_beep(app):
    """test get busser beep"""
    request, response = app.test_client.get("http://127.0.0.1:5823/busser_beep")
    assert response.status == 200

def test_get_media(app):
    """test get media url"""
    request, response = app.test_client.get("http://127.0.0.1:5823/get_media")
    assert response.status == 200


def test_camera_off(app):
    """test camera off"""
    request, response = app.test_client.get("http://127.0.0.1:5823/camera_off")
    assert response.status == 200

# def test_import_c(app):
#     """test import_c to python"""
#     request, response = app.test_client.get("http://127.0.0.1:5823/import_c")
#     assert response.status == 200

def test_derived_key(app):
    """test getting derived_key"""
    request, response = app.test_client.get("http://127.0.0.1:5823/derived_key")
    assert response.status == 200

def test_same_derived_key(app):
    """test getting same_derived_key"""
    request, response = app.test_client.get("http://127.0.0.1:5823/same_derived_key")
    assert response.status == 200


""" Application initializer """
from sanic import Sanic
from sanic_class import DerivedKeyRoutes, SameDerivedKeyRoutes, SetUpRoutes, RequestIdRoutes, StatusRoutes, CreatedAtRoutes, IdRoutes, CameraOnRoutes, \
    ImportCLanguage, CameraCloseRoutes, ChangeLedRoutes, BusserBeepRoutes, GetMediaRoutes
import logging

app = Sanic(__name__)

Log_Format = "%(levelname)s %(asctime)s - %(message)s"

logging.basicConfig(filename = "logfile.log",
                    filemode = "w",
                    format = Log_Format,
                    level = logging.ERROR)

# def create_app():
#     app = Sanic(__name__)
#     return app

app.add_route(SetUpRoutes.as_view(), "/gets")
app.add_route(IdRoutes.as_view(), "/get/<id_:int>")
app.add_route(RequestIdRoutes.as_view(), "/request_id/<request_id:str>")
app.add_route(StatusRoutes.as_view(), "/status/<status:str>")
app.add_route(CreatedAtRoutes.as_view(), "/created_at/<created_at:str>")
app.add_route(CameraOnRoutes.as_view(), "/camera_on")
app.add_route(CameraCloseRoutes.as_view(), "/camera_off")
app.add_route(ImportCLanguage.as_view(), "/import_c")
app.add_route(ChangeLedRoutes.as_view(), "/change_led")
app.add_route(BusserBeepRoutes.as_view(), "/busser_beep")
app.add_route(GetMediaRoutes.as_view(), "/get_media")
app.add_route(DerivedKeyRoutes.as_view(), "/derived_key")
app.add_route(SameDerivedKeyRoutes.as_view(), "/same_derived_key")



# exception_handler(app=app)

if __name__ == "__main__":
    app.run(port=2000, workers=1)


